# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from aws_embedded_metrics.logger.metrics_context import MetricsContext


class Sink(abc.ABC):
    """The mechanism by which logs are sent to their destination."""

    @staticmethod
    @abc.abstractmethod
    def name() -> str:
        """The name of the sink."""

    @abc.abstractmethod
    def accept(self, context: MetricsContext) -> None:
        """Flushes the metrics context to the sink."""


class SocketClient(abc.ABC):
    """Interface for pushing data to a socket"""

    @abc.abstractmethod
    def send_message(self, message: bytes) -> None:
        """Send binary payload to socket"""
