# Copyright 2019 Amazon.com, Inc. or its affiliates.
# Licensed under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from aws_embedded_metrics.config.configuration import Configuration
from aws_embedded_metrics.config.environment_configuration_provider import (
    EnvironmentConfigurationProvider,
)

config = EnvironmentConfigurationProvider().get_configuration()


def get_config() -> Configuration:
    """Gets the current configuration"""
    return config
